#!/usr/bin/env python3
# vim: ft=python

import argparse
import sys
import logging
import math
import fileinput

from pwnedpasswords import Password
from pwnedpasswords import PasswordNotFound


class PwnedPasswordsCommandLineHandler(object):
    def __init__(self, namespace):
        self.plain_text = namespace.plain_text
        self.verbosity = logging.DEBUG if namespace.verbose else logging.WARNING

        # Configure logging handler for verbose output
        if namespace.verbose:
            logging.basicConfig(
                level=logging.DEBUG,
                format='%(levelname)s:%(name)s:%(message)s'
            )

        try:
            self.password = Password(
                namespace.password, self.plain_text, verbosity=self.verbosity
            )
        except Exception:
            self.password = None

    def check_single_password(self, password):
        try:
            value = password.check()
        except PasswordNotFound:
            print("Password not found.")
            value = 0
        else:
            print(value)
        return value

    def run(self, *args):
        if self.password is None:
            try:
                for line in fileinput.input(files=["-"]):
                    password = Password(
                        line.strip(), self.plain_text, verbosity=self.verbosity
                    )
                    value = self.check_single_password(password)
            except KeyboardInterrupt:
                print("Quitting")
        else:
            value = self.check_single_password(self.password)

        if value > 0:
            # Most systems require exit status to be in the range 0-127, so
            # return 1 plus the base-10 log of the number of entries instead of
            # the full number of entries. This should be handy for scripts and
            # other CLI utilities that need to parse this output.
            sys.exit(1 + int(math.log10(value)))
        else:
            sys.exit(None)


def BoolAction(true_choice):
    class Action(argparse.Action):
        def __call__(self, parser, namespace, values, option_string=None):
            setattr(namespace, self.dest, option_string == true_choice)

    return Action


def main():
    parser = argparse.ArgumentParser(
        prog="pwnedpasswords",
        description="Checks Pwned Passwords API to see if provided plaintext data was found in a data breach.",
    )
    parser.add_argument(
        "--verbose",
        nargs=0,
        action=BoolAction("--verbose"),
        dest="verbose",
        default=False,
        help="Display verbose output.",
    )
    parser.add_argument(
        "--plain-text",
        nargs=0,
        action=BoolAction("--plain-text"),
        dest="plain_text",
        default=True,
        help="Specify that the provided input is plain text, even if it looks like a SHA-1 hash.",
    )

    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        "--stdin",
        nargs=0,
        action=BoolAction("--stdin"),
        dest="stdin",
        help="Read provided input from stdin.",
        default=False,
    )
    group.add_argument(
        "password",
        type=str,
        nargs="?",
        help="The password or hashed password to search for.",
    )

    namespace = parser.parse_args()
    handler = PwnedPasswordsCommandLineHandler(namespace)
    handler.run()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# Copyright 2018 Lionheart Software LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .metadata import (
    __author__,
    __copyright__,
    __email__,
    __license__,
    __maintainer__,
    __version__,
)

from .pwnedpasswords import Password
from .pwnedpasswords import check
from .exceptions import BadRequest
from .exceptions import NoUserAgent
from .exceptions import PasswordNotFound

__all__ = [
    "__author__",
    "__copyright__",
    "__email__",
    "__license__",
    "__maintainer__",
    "__version__",
    "Password",
    "BadRequest",
    "NoUserAgent",
    "PasswordNotFound",
    "RateLimitExceededchecksearchrange",
    "check",
]



#!/usr/bin/env python3

# Copyright 2018 Lionheart Software LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import hashlib
import logging
import re
import urllib.error
import urllib.request
from . import exceptions

looks_like_sha1_re = re.compile(r"^[a-fA-F0-9]{40}")

logger = logging.getLogger(__name__)


def check(password, plain_text=False, timeout=None, anonymous=True):
    password = Password(password, plain_text=plain_text)
    return password.check(timeout=timeout, anonymous=anonymous)


def search(password, plain_text=False, timeout=None):
    password = Password(password, plain_text=plain_text)
    return password.search(timeout=timeout)


def range(password, plain_text=False, timeout=None):
    password = Password(password, plain_text=plain_text)
    return password.range(timeout=timeout)


class PwnedPasswordsAPI(object):
    @staticmethod
    def url(*components, **kwargs):
        value = "https://api.pwnedpasswords.com/" + "/".join(components)
        if len(kwargs) > 0:
            value += "?" + urllib.parse.urlencode(kwargs)

        logger.info(value)
        return value

    @staticmethod
    def request(path, value, timeout=None, **kwargs):
        url = PwnedPasswordsAPI.url(path, value, **kwargs)
        request = urllib.request.Request(
            url=url, headers={"User-Agent": "pwnedpasswords (Python)"}
        )
        try:
            with urllib.request.urlopen(request, timeout=timeout) as f:
                response = f.read()
        except urllib.error.HTTPError as e:
            logger.debug("Exception found: {}".format(e))
            Exception = exceptions.STATUS_CODES_TO_EXCEPTIONS.get(e.code)
            if Exception is not None:
                exception = Exception(e.url, e.code, e.msg, e.hdrs, e.fp)
                raise exception from e
            else:
                raise
        else:
            return response.decode("utf-8-sig")


def convert_password_tuple(value):
    hash, count = value.split(":")
    return (hash, int(count))


class Password(object):
    def __init__(self, value, plain_text=False, verbosity=logging.WARNING):
        logger.setLevel(verbosity)

        if looks_like_sha1_re.match(value) and not plain_text:
            self.value = value
        else:
            # The provided value is plaintext, so let's hash it. If you'd like
            # to search the provided value as-is, specify `raw=True` in the
            # initializer.
            self.value = hashlib.sha1(value.encode("utf8")).hexdigest()

    def check(self, timeout=None, anonymous=True):
        if anonymous:
            entries = self.range(timeout=timeout)
            entry = entries.get(self.value[5:].upper())
            if entry is None:
                logger.info("No entry found, returning 0")
                return 0
            else:
                logger.info("Entry found")
                return entry
        else:
            return self.search(timeout=timeout)

    def search(self, timeout=None):
        try:
            kwargs = {}
            if timeout:
                kwargs["timeout"] = timeout

            response = PwnedPasswordsAPI.request("pwnedpassword", self.value, **kwargs)
        except exceptions.PasswordNotFound:
            logger.info("No password found")
            return 0
        else:
            logger.info("Password found")
            count = int(response)
            return count

    def range(self, timeout=None):
        response = PwnedPasswordsAPI.request(
            "range", self.value[:5].upper(), timeout=timeout
        )
        entries = dict(map(convert_password_tuple, response.upper().split("\r\n")))
        return entries



#!/usr/bin/env/python
# -*- coding: utf-8 -*-

# Copyright 2018 Lionheart Software LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import unittest
import runpy
from setuptools import Command, setup

metadata_filename = "pwnedpasswords/metadata.py"
metadata = runpy.run_path(metadata_filename)

with open(os.path.join(os.path.dirname(__file__), "README.md")) as file:
    long_description = file.read()

classifiers = [
    "Development Status :: 5 - Production/Stable",
    "Environment :: Console",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: Apache Software License",
    "Natural Language :: English",
    "Operating System :: OS Independent",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Programming Language :: Python :: 3.11",
    "Programming Language :: Python :: 3.12",
    "Programming Language :: Python :: 3.13",
    "Programming Language :: Python",
    "Topic :: Internet :: WWW/HTTP :: Dynamic Content :: CGI Tools/Libraries",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Topic :: Utilities",
]


class TestCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from test_pwnedpasswords import TestPwnedPasswords

        suite = unittest.TestLoader().loadTestsFromTestCase(TestPwnedPasswords)
        unittest.TextTestRunner(verbosity=2).run(suite)


setup(
    author=metadata["__author__"],
    author_email=metadata["__email__"],
    classifiers=classifiers,
    description="A Python wrapper for Troy Hunt's Pwned Passwords API.",
    install_requires=[],
    keywords="passwords security",
    license=metadata["__license__"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    name="pwnedpasswords",
    package_data={"": ["LICENSE", "README.rst"]},
    packages=["pwnedpasswords"],
    url="https://github.com/lionheart/pwnedpasswords",
    download_url="https://github.com/lionheart/pwnedpasswords/tarball/{}".format(
        metadata["__version__"]
    ),
    version=metadata["__version__"],
    entry_points={
        "console_scripts": [
            "pwnedpasswords=pwnedpasswords.__main__:main",
        ],
    },
    test_suite="nose.collector",
    tests_require=["nose"],
)
